@@ -3,7 +3,14 @@ from ._version import get_versions
 __version__ = get_versions()["version"]
 del get_versions
 
-from .manager.comms import ZMQCommSendAsync, ZMQCommSendThreads, CommTimeoutError  # noqa: E402, F401
+from .manager.comms import (  # noqa: E402, F401
+    ZMQCommSendAsync,
+    ZMQCommSendThreads,
+    CommTimeoutError,
+    generate_zmq_keys,
+    generate_zmq_public_key,
+    validate_zmq_key,
+)
 from .manager.annotation_decorator import parameter_annotation_decorator  # noqa: E402, F401
 from .manager.output_streaming import ReceiveConsoleOutput, ReceiveConsoleOutputAsync  # noqa: E402, F401
 
